#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/event.h>
#include <sys/time.h>
#include <time.h>
#include <netdb.h>
#include <errno.h>
#include "netfd.h"

#define	DEF_ADDR	"192.168.1.23"
#define	DEF_PORT	"27182"
#define	DEF_BACKLOG	(2)

static void	echo(netfd_t *);
static void	info(netfd_t *);

static void
echo(nfd)
	netfd_t *	nfd;
{
  char		buf[BUFSIZ];
  ssize_t	n;

  memset(buf, 0, sizeof(buf));
  n = netfd_readline(nfd, buf, sizeof(buf), NETFD_NO_TIMEOUT);
  if (n > 0) {
    fprintf(stderr, "DEBUG[echo] %s", buf);
    netfd_write(nfd, buf, strlen(buf), NETFD_NO_TIMEOUT);
  }
  if (strncmp(buf, "quit", 4) == 0) {
    netfd_close(nfd);
  }
}

static void
info(nfd)
	netfd_t *	nfd;
{
  char	host[NI_MAXHOST];
  char	serv[NI_MAXSERV];

  if (netfd_getnameinfo(nfd, host, sizeof(host), serv, sizeof(serv)) == 0) {
    fprintf(stderr, "DEBUG[info] %s:%s\n", host, serv);
  }
}

int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  if (netfd_init(0) == 0) {
    netfd_t *	nsd;

    nsd = netfd_tcp_server(DEF_ADDR, DEF_PORT, DEF_BACKLOG);
    if (nsd != NULL) {
      int	kq;

      fprintf(stderr, "DEBUG: %s:%s\n", DEF_ADDR, DEF_PORT);
      kq = kqueue();
      if (kq != -1) {
	int		nev;
	struct kevent	kev[10];

	EV_SET(&kev[0], netfd_fileno(nsd), EVFILT_READ, EV_ADD, 0, 0, nsd);
	nev = 1;

	for (;;) {
	  int	n;
	  int	i;

	  n = kevent(kq, kev, nev, kev, sizeof(kev)/sizeof(kev[0]), NULL);
	  nev = 0;
	  for (i = 0; i < n; i++) {
	    netfd_t *	nfd = (netfd_t *)kev[i].udata;

	    fprintf(stderr, "DEBUG: flags=0x%04x data=%d\n", kev[i].flags, kev[i].data);
	    if (nfd->flags == NETFD_FL_LISTENER) {
	      struct sockaddr 	cliaddr;
	      socklen_t		clilen;
	      netfd_t *		newfd;

	      newfd = netfd_accept(nfd, &cliaddr, &clilen, NETFD_NO_TIMEOUT);
	      if (newfd != NULL) {
		EV_SET(&kev[nev++], netfd_fileno(newfd), EVFILT_READ, EV_ADD,
		       0, 0, newfd);
	      }
	    }
	    else if (kev[i].flags == EV_EOF) {
	      netfd_close(nfd);
	    }
	    else {
	      info(nfd);
	      echo(nfd);
	    }
	  }
	}
      }
      netfd_close(nsd);
    }
    netfd_finish();
  }

  return 0;
}
